<?php
namespace MyApi\Model\ExternalApi\MyBrand;

use Catalog\Model\Orm\Brand;
use ExternalApi\Model\AbstractMethods\AbstractAuthorizedMethod;
use ExternalApi\Model\Exception as ApiException;
use ExternalApi\Model\Utils as ExternalApiUtils;
use RS\Orm\AbstractObject;

/**
 * Класс описывает метод внешнего API MyBrand.get
 */
class Get extends AbstractAuthorizedMethod
{
    /**
     * Право на чтение
     */
    const RIGHT_READ = 'read';

    /**
     * Возвращает комментарии к кодам прав доступа.
     * Тексовое обозначение права доступа будет использоваться в сообщениях об ошибках.
     * Метод можно будет запустить только если у приложения есть хотя бы одно из перечисленных здесь прав.
     *
     * @return [
     *     КОД => КОММЕНТАРИЙ,
     *     КОД => КОММЕНТАРИЙ,
     *     ...
     * ]
     */
    public function getRightTitles()
    {
        return [
            self::RIGHT_READ => t('Право на чтение данных')
        ];
    }

    /**
     * Возвращает какой объект нужно загружать
     *
     * @return AbstractObject
     */
    public function getOrmObject()
    {
        return new Brand();
    }

    /**
     * Возвращает объект бренда с полными сведениями
     *
     * @param string $token Токен авторизации
     * @param integer $id ID бренда
     *
     * @return array
     * @example GET /api/methods/myBrand.get?token=48081b693d3c239071e5a3d2a28a4a35&id=1
     * <pre>
     * {
     *       "response": {
     *           "brand": {
     *               "id": "1",
     *               "title": "Asus",
     *               "alias": "asus",
     *               "public": "1",
     *               "image": {
     *                   "original_url": "http://full.readyscript.local/storage/system/original/39071e5a3d2a28a4a3548081b693d3c2.png",
     *                   "big_url": "http://full.readyscript.local/storage/system/resized/xy_1000x1000/39071e5a3d2a28a4a3548081b693d3c2_9e4a09f4.png",
     *                   "middle_url": "http://full.readyscript.local/storage/system/resized/xy_600x600/39071e5a3d2a28a4a3548081b693d3c2_d47e16ce.png",
     *                   "small_url": "http://full.readyscript.local/storage/system/resized/xy_300x300/39071e5a3d2a28a4a3548081b693d3c2_bda7f00c.png",
     *                   "micro_url": "http://full.readyscript.local/storage/system/resized/xy_100x100/39071e5a3d2a28a4a3548081b693d3c2_5cb79af2.png",
     *                   "nano_url": "http://full.readyscript.local/storage/system/resized/xy_50x50/39071e5a3d2a28a4a3548081b693d3c2_8467bf54.png"
     *               },
     *               "description": "Расположенная на Тайване компания, производящая персональные компьютеры...",
     *               "xml_id": null,
     *               "meta_title": "",
     *               "meta_keywords": "",
     *               "meta_description": "",
     *               "meta_custom": "",
     *               "meta_image": "",
     *               "meta_html_description": null
     *           }
     *       }
     *   }
     * </pre>
     * @throws ApiException
     */
    protected function process($token, $id)
    {
        //Параметр $token - обязательно перечислить в аргументах. Он будет обработан в AbstractAuthorizedMethod

        $this->object = $this->getOrmObject(); //Получаем объект
        if ($this->object->load($id)) { //Загружаем объект

            return [
                'response' => [
                    'brand' => ExternalApiUtils::extractOrm($this->object) //Конвертируем ORM объект в массив значений
                ]
            ];

        }

        throw new ApiException(t('Объект с таким ID не найден'), ApiException::ERROR_OBJECT_NOT_FOUND);
    }
}