<?php
namespace MyApi\Model\ExternalApi\MyBrand;

use Catalog\Model\BrandApi;
use ExternalApi\Model\AbstractMethods\AbstractMethod;
use ExternalApi\Model\Utils as ExternalApiUtils;
use RS\Module\AbstractModel\EntityList;

/**
 * Класс описывает метод внешнего API MyBrand.getList
 */
class GetList extends AbstractMethod
{
    protected $dao;

    /**
     * Возвращает список брендов, присутствующих в системе. Информация по каждому бренду ограничивается: ID, Названием, ссылкой на картинку
     *
     * @param integer $page Номер страницы
     * @param integer $pageSize Количество элементов на странице
     * @return array
     * @example GET /api/methods/mybrand.getlist
     * <pre>
     * {
     *   "response": {
     *       "summary": {
     *           "page": 1,
     *           "pageSize": 2,
     *           "total": "14"
     *       },
     *       "list": [
     *           {
     *               "id": "16",
     *               "title": "Asus",
     *               "image": {
     *                   "original_url": "http://full.readyscript.local/storage/system/original/81676002a34107daded7349815fcd163.jpg",
     *                   "big_url": "http://full.readyscript.local/storage/system/resized/xy_1000x1000/81676002a34107daded7349815fcd163_4428c733.jpg",
     *                   "middle_url": "http://full.readyscript.local/storage/system/resized/xy_600x600/81676002a34107daded7349815fcd163_cffa770d.jpg",
     *                   "small_url": "http://full.readyscript.local/storage/system/resized/xy_300x300/81676002a34107daded7349815fcd163_a62391cf.jpg",
     *                   "micro_url": "http://full.readyscript.local/storage/system/resized/xy_100x100/81676002a34107daded7349815fcd163_4733fb31.jpg",
     *                   "nano_url": "http://full.readyscript.local/storage/system/resized/xy_50x50/81676002a34107daded7349815fcd163_25ef3206.jpg"
     *               }
     *           },
     *           {
     *               "id": "14",
     *               "title": "НP",
     *               "image": null
     *           }
     *       ]
     *   }
     *}
     * </pre>
     *
     */
    protected function process($page = 1, $pageSize = 20)
    {
        //Специально дробим выорку на мелкие функции getDaoObject, setFilter, setOrder....,
        //так текущий класс будет возможно использовать в качестве базового для какой-нибудь другой
        //выборки в других классах. Нужно будет только переопределить некоторые методы.

        $this->dao = $this->getDaoObject();
        $this->setFilter($this->dao, [
            'public' => 1
        ]);
        $this->setOrder($this->dao, 'id desc');

        return [
            'response' => [
                'summary' => [
                    'page' => $page,
                    'pageSize' => $pageSize,
                    'total' => $this->getResultCount($this->dao),
                ],
                'list' => $this->getResultList($this->dao, $page, $pageSize),
            ]
        ];
    }

    /**
     * Возвращает DAO (Data Access Object) объект, с помощью которого можн овыбирать ORM объекты.
     * ORM объекты - это объекты, которые описывают сущность, представленную в БД (например, Бренд, Товар, и т.д.)
     *
     * @return BrandApi Возвращает наследника EntityList
     */
    protected function getDaoObject()
    {
        return new BrandApi();
    }

    /**
     * Устанавливает фильтр на выборку объектов
     *
     * @param EntityList $dao DAO объект
     * @param array $filter массив с фильтрами
     */
    protected function setFilter($dao, $filter)
    {
        $dao->setFilter($filter);
    }

    /**
     * Устанавливает сортировку для выборки ORM объектов
     *
     * @param EntityList $dao DAO объект
     * @param string $sort Поле сортировки и направление, например "id DESC"
     */
    protected function setOrder($dao, $sort)
    {
        $dao->setOrder($sort);
    }

    /**
     * Возвращает общее количество элементов, доступное для выборки
     *
     * @param EntityList $dao DAO объект
     * @return integer
     */
    protected function getResultCount($dao)
    {
        return $dao->getListCount();
    }

    /**
     * Возвращает список объектов
     *
     * @param EntityList $dao DAO объект
     * @param integer $page Номер страницы
     * @param integer $pageSize Количество элементов на страницу
     * @return array
     */
    public function getResultList($dao, $page, $pageSize)
    {
        $result = [];
        foreach($dao->getList($page, $pageSize) as $orm_object) {
            $result[] = array_intersect_key( //Отфильтруем только нужные нам поля
                ExternalApiUtils::extractOrm($orm_object), //Получим массив со всеми полями ORM-объекта
                array_flip(['id', 'title', 'image'])
            );

        }
        return $result;
    }
}